/*------------------------------------------------------------------------------
*                      Create Groups.
*-----------------------------------------------------------------------------*/

IF NOT EXISTS (
	SELECT 1
	FROM SYS.SYSUSER
	WHERE user_name = 'Person'
) THEN
	GRANT CONNECT TO "Person";
	GRANT GROUP TO "Person";
END IF;

IF NOT EXISTS (
	SELECT 1
	FROM SYS.SYSUSER
	WHERE user_name = 'Sales'
) THEN
	GRANT CONNECT TO "Sales";
	GRANT GROUP TO "Sales";
END IF;

IF NOT EXISTS (
	SELECT 1
	FROM SYS.SYSUSER
	WHERE user_name = 'Production'
) THEN
	GRANT CONNECT TO "Production";
	GRANT GROUP TO "Production";
END IF;

IF NOT EXISTS (
	SELECT 1
	FROM SYS.SYSUSER
	WHERE user_name = 'Purchasing'
) THEN
	GRANT CONNECT TO "Purchasing";
	GRANT GROUP TO "Purchasing";
END IF;

COMMIT;

/*------------------------------------------------------------------------------
*                 Create table 'AdventureWorks.Person.Address'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'Address'
	AND user_name = 'Person'
) THEN
	DROP TABLE "Person"."Address"
END IF;
CREATE TABLE "Person"."Address"     (
	"AddressID" integer not null,
	"AddressLine1" nvarchar(60) not null,
	"AddressLine2" nvarchar(60) null,
	"City" nvarchar(30) not null,
	"StateProvinceID" integer not null,
	"PostalCode" nvarchar(15) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("AddressID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Person.AddressType'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'AddressType'
	AND user_name = 'Person'
) THEN
	DROP TABLE "Person"."AddressType"
END IF;
CREATE TABLE "Person"."AddressType"     (
	"AddressTypeID" integer not null,
	"Name" nvarchar(50) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("AddressTypeID")
);

COMMIT;
/*------------------------------------------------------------------------------
*                 Create table 'AdventureWorks.Person.Contact'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'Contact'
	AND user_name = 'Person'
) THEN
	DROP TABLE "Person"."Contact"
END IF;
CREATE TABLE "Person"."Contact"     (
	"ContactID" integer not null,
	"NameStyle" bit not null,
	"Title" nvarchar(8) null,
	"FirstName" nvarchar(50) not null,
	"MiddleName" nvarchar(50) null,
	"LastName" nvarchar(50) not null,
	"Suffix" nvarchar(10) null,
	"EmailAddress" nvarchar(50) null,
	"EmailPromotion" integer not null,
	"Phone" nvarchar(25) null,
	"PasswordHash" varchar(128) not null,
	"PasswordSalt" varchar(10) not null,
	"AdditionalContactInfo" xml null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("ContactID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Person.ContactType'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'ContactType'
	AND user_name = 'Person'
) THEN
	DROP TABLE "Person"."ContactType"
END IF;
CREATE TABLE "Person"."ContactType"     (
	"ContactTypeID" integer not null,
	"Name" nvarchar(50) not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("ContactTypeID")
);

COMMIT;
/*------------------------------------------------------------------------------
*                 Create table 'AdventureWorks.Sales.Customer'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'Customer'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."Customer"
END IF;
CREATE TABLE "Sales"."Customer"     (
	"CustomerID" integer not null,
	"TerritoryID" integer null,
	"AccountNumber" varchar(10) not null,
	"CustomerType" nchar(1) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("CustomerID")
);

COMMIT;
/*------------------------------------------------------------------------------
*             Create table 'AdventureWorks.Sales.CustomerAddress'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'CustomerAddress'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."CustomerAddress"
END IF;
CREATE TABLE "Sales"."CustomerAddress"     (
	"CustomerID" integer not null,
	"AddressID" integer not null,
	"AddressTypeID" integer not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("CustomerID", "AddressID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Production.Product'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'Product'
	AND user_name = 'Production'
) THEN
	DROP TABLE "Production"."Product"
END IF;
CREATE TABLE "Production"."Product"     (
	"ProductID" integer not null,
	"Name" nvarchar(50) not null,
	"ProductNumber" nvarchar(25) not null,
	"MakeFlag" bit not null,
	"FinishedGoodsFlag" bit not null,
	"Color" nvarchar(15) null,
	"SafetyStockLevel" smallint not null,
	"ReorderPoint" smallint not null,
	"StandardCost" numeric(19, 4) not null,
	"ListPrice" numeric(19, 4) not null,
	"Size" nvarchar(5) null,
	"SizeUnitMeasureCode" nchar(3) null,
	"WeightUnitMeasureCode" nchar(3) null,
	"Weight" decimal(8, 2) null,
	"DaysToManufacture" integer not null,
	"ProductLine" nchar(2) null,
	"Class" nchar(2) null,
	"Style" nchar(2) null,
	"ProductSubcategoryID" integer null,
	"ProductModelID" integer null,
	"SellStartDate" timestamp not null,
	"SellEndDate" timestamp null,
	"DiscontinuedDate" timestamp null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("ProductID")
);

COMMIT;
/*------------------------------------------------------------------------------
*             Create table 'AdventureWorks.Sales.SalesOrderDetail'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'SalesOrderDetail'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."SalesOrderDetail"
END IF;
CREATE TABLE "Sales"."SalesOrderDetail"     (
	"SalesOrderID" integer not null,
	"SalesOrderDetailID" integer not null,
	"CarrierTrackingNumber" nvarchar(25) null,
	"OrderQty" smallint not null,
	"ProductID" integer not null,
	"SpecialOfferID" integer not null,
	"UnitPrice" numeric(19, 4) not null,
	"UnitPriceDiscount" numeric(19, 4) not null,
	"LineTotal" numeric(38, 6) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("SalesOrderID", "SalesOrderDetailID")
);

COMMIT;
/*------------------------------------------------------------------------------
*             Create table 'AdventureWorks.Sales.SalesOrderHeader'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'SalesOrderHeader'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."SalesOrderHeader"
END IF;
CREATE TABLE "Sales"."SalesOrderHeader"     (
	"SalesOrderID" integer not null,
	"RevisionNumber" tinyint not null,
	"OrderDate" timestamp not null,
	"DueDate" timestamp not null,
	"ShipDate" timestamp null,
	"Status" tinyint not null,
	"OnlineOrderFlag" bit not null,
	"SalesOrderNumber" nvarchar(25) not null,
	"PurchaseOrderNumber" nvarchar(25) null,
	"AccountNumber" nvarchar(15) null,
	"CustomerID" integer not null,
	"ContactID" integer not null,
	"TerritoryID" integer null,
	"BillToAddressID" integer not null,
	"ShipToAddressID" integer not null,
	"ShipMethodID" integer not null,
	"CreditCardID" integer null,
	"CreditCardApprovalCode" varchar(15) null,
	"CurrencyRateID" integer null,
	"SubTotal" numeric(19, 4) not null,
	"TaxAmt" numeric(19, 4) not null,
	"Freight" numeric(19, 4) not null,
	"TotalDue" numeric(19, 4) not null,
	"Comment" nvarchar(128) null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("SalesOrderID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Sales.SalesPerson'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'SalesPerson'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."SalesPerson"
END IF;
CREATE TABLE "Sales"."SalesPerson"     (
	"SalesPersonID" integer not null,
	"TerritoryID" integer null,
	"SalesQuota" numeric(19, 4) null,
	"Bonus" numeric(19, 4) not null,
	"CommissionPct" numeric(10, 4) not null,
	"SalesYTD" numeric(19, 4) not null,
	"SalesLastYear" numeric(19, 4) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("SalesPersonID")
);

COMMIT;
/*------------------------------------------------------------------------------
*             Create table 'AdventureWorks.Purchasing.ShipMethod'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'ShipMethod'
	AND user_name = 'Purchasing'
) THEN
	DROP TABLE "Purchasing"."ShipMethod"
END IF;
CREATE TABLE "Purchasing"."ShipMethod"     (
	"ShipMethodID" integer not null,
	"Name" nvarchar(50) not null,
	"ShipBase" numeric(19, 4) not null,
	"ShipRate" numeric(19, 4) not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("ShipMethodID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Sales.SpecialOffer'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'SpecialOffer'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."SpecialOffer"
END IF;
CREATE TABLE "Sales"."SpecialOffer"     (
	"SpecialOfferID" integer not null,
	"Description" nvarchar(255) not null,
	"DiscountPct" numeric(10, 4) not null,
	"Type" nvarchar(50) not null,
	"Category" nvarchar(50) not null,
	"StartDate" timestamp not null,
	"EndDate" timestamp not null,
	"MinQty" integer not null,
	"MaxQty" integer null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("SpecialOfferID")
);

COMMIT;
/*------------------------------------------------------------------------------
*           Create table 'AdventureWorks.Sales.SpecialOfferProduct'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'SpecialOfferProduct'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."SpecialOfferProduct"
END IF;
CREATE TABLE "Sales"."SpecialOfferProduct"     (
	"SpecialOfferID" integer not null,
	"ProductID" integer not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("SpecialOfferID", "ProductID")
);

COMMIT;
/*------------------------------------------------------------------------------
*              Create table 'AdventureWorks.Person.StateProvince'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'StateProvince'
	AND user_name = 'Person'
) THEN
	DROP TABLE "Person"."StateProvince"
END IF;
CREATE TABLE "Person"."StateProvince"     (
	"StateProvinceID" integer not null,
	"StateProvinceCode" nchar(3) not null,
	"CountryRegionCode" nvarchar(3) not null,
	"IsOnlyStateProvinceFlag" bit not null,
	"Name" nvarchar(50) not null,
	"TerritoryID" integer not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("StateProvinceID")
);

COMMIT;
/*------------------------------------------------------------------------------
*                  Create table 'AdventureWorks.Sales.Store'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'Store'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."Store"
END IF;
CREATE TABLE "Sales"."Store"     (
	"CustomerID" integer not null,
	"Name" nvarchar(50) not null,
	"SalesPersonID" integer null,
	"Demographics" xml null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("CustomerID")
);

COMMIT;
/*------------------------------------------------------------------------------
*               Create table 'AdventureWorks.Sales.StoreContact'.
*-----------------------------------------------------------------------------*/
IF EXISTS (
	SELECT 1
	FROM SYS.SYSTABLE
	INNER JOIN SYS.SYSUSER ON user_id = creator
	WHERE table_name = 'StoreContact'
	AND user_name = 'Sales'
) THEN
	DROP TABLE "Sales"."StoreContact"
END IF;
CREATE TABLE "Sales"."StoreContact"     (
	"CustomerID" integer not null,
	"ContactID" integer not null,
	"ContactTypeID" integer not null,
	"rowguid" uniqueidentifier not null,
	"ModifiedDate" timestamp not null,
	PRIMARY KEY ("CustomerID", "ContactID")
);

COMMIT;


COMMIT;
