/*------------------------------------------------------------------------------
*                       Delete subscription to 'publication' for all.
*-----------------------------------------------------------------------------*/

IF EXISTS ( SELECT 1 FROM SYS.SYSSYNCSUBSCRIPTIONS WHERE publication_name = 'publication' ) THEN
	DROP SYNCHRONIZATION SUBSCRIPTION TO "publication" UNCONDITIONALLY;
END IF;

/*------------------------------------------------------------------------------
*                       Delete publication 'publication'.
*-----------------------------------------------------------------------------*/

IF EXISTS ( SELECT 1 FROM SYS.SYSPUBLICATIONS WHERE publication_name = 'publication' ) THEN
	DROP PUBLICATION "publication";
END IF;

/*------------------------------------------------------------------------------
*                       Delete trigger 'iduSalesOrderDetail'.
*-----------------------------------------------------------------------------*/

IF EXISTS ( SELECT 1 FROM SYS.SYSTRIGGERS WHERE trigname = 'iduSalesOrderDetail' ) THEN
	DROP TRIGGER iduSalesOrderDetail;
END IF;

/*------------------------------------------------------------------------------
*                       Delete procedure 'sp_Report'.
*-----------------------------------------------------------------------------*/

IF EXISTS ( SELECT 1 FROM SYS.SYSPROCEDURE WHERE proc_name = 'sp_Report' ) THEN
	DROP PROCEDURE "SalesPerson".sp_Report;
END IF;

/*------------------------------------------------------------------------------
*                       Create 'SalesPerson' Group
*-----------------------------------------------------------------------------*/

GRANT CONNECT TO "SalesPerson";
GRANT GROUP TO "SalesPerson";

/*------------------------------------------------------------------------------
*                       Alter Defaulted & Calculated Columns.
*-----------------------------------------------------------------------------*/

ALTER TABLE "Sales"."SalesOrderHeader" ALTER "SalesOrderID" SET DEFAULT global autoincrement( 100000 );
ALTER TABLE "Sales"."SalesOrderHeader" ALTER "SalesOrderNumber" SET COMPUTE( (isnull('SO'+CONVERT(nvarchar(23),SalesOrderID,0),'*** ERROR ***')) );
ALTER TABLE "Sales"."SalesOrderHeader" ALTER "TotalDue" SET COMPUTE( (isnull((SubTotal+TaxAmt)+Freight,(0))) );
ALTER TABLE "Sales"."SalesOrderHeader" ALTER "rowguid" SET DEFAULT newid();
ALTER TABLE "Sales"."SalesOrderHeader" ALTER "ModifiedDate" SET DEFAULT timestamp;
ALTER PRIMARY KEY ON "Sales"."SalesOrderHeader" RENAME TO "SalesOrderHeader";

ALTER TABLE "Sales"."SalesOrderDetail" ALTER "SalesOrderDetailID" SET DEFAULT autoincrement;
ALTER TABLE "Sales"."SalesOrderDetail" ALTER "LineTotal" SET COMPUTE( (isnull((UnitPrice*((1.0)-UnitPriceDiscount))*OrderQty,(0.0))) );
ALTER TABLE "Sales"."SalesOrderDetail" ALTER "rowguid" SET DEFAULT newid();
ALTER TABLE "Sales"."SalesOrderDetail" ALTER "ModifiedDate" SET DEFAULT timestamp;
ALTER PRIMARY KEY ON "Sales"."SalesOrderDetail" RENAME TO "SalesOrderDetail";

/*------------------------------------------------------------------------------
*                       Create Foreign Key Relationships.
*-----------------------------------------------------------------------------*/

ALTER TABLE "Person"."Address" ADD CONSTRAINT "StateProvince" NOT NULL FOREIGN KEY ( "StateProvinceID" ASC ) REFERENCES "Person"."StateProvince" ( "StateProvinceID" );


ALTER TABLE "Sales"."CustomerAddress" ADD CONSTRAINT "Customer" NOT NULL FOREIGN KEY ( "CustomerID" ASC ) REFERENCES "Sales"."Customer" ( "CustomerID" );
ALTER TABLE "Sales"."CustomerAddress" ADD CONSTRAINT "Address" NOT NULL FOREIGN KEY ( "AddressID" ASC ) REFERENCES "Person"."Address" ( "AddressID" );
ALTER TABLE "Sales"."CustomerAddress" ADD CONSTRAINT "AddressType" NOT NULL FOREIGN KEY ( "AddressTypeID" ASC ) REFERENCES "Person"."AddressType" ( "AddressTypeID" );

ALTER TABLE "Sales"."SalesOrderHeader" ADD CONSTRAINT "BillToAddress" NOT NULL FOREIGN KEY ( "BillToAddressID" ASC ) REFERENCES "Person"."Address" ( "AddressID" );
ALTER TABLE "Sales"."SalesOrderHeader" ADD CONSTRAINT "ShipToAddress" NOT NULL FOREIGN KEY ( "ShipToAddressID" ASC ) REFERENCES "Person"."Address" ( "AddressID" );
ALTER TABLE "Sales"."SalesOrderHeader" ADD CONSTRAINT "Contact" NOT NULL FOREIGN KEY ( "ContactID" ASC ) REFERENCES "Person"."Contact" ( "ContactID" );
ALTER TABLE "Sales"."SalesOrderHeader" ADD CONSTRAINT "Customer" NOT NULL FOREIGN KEY ( "CustomerID" ASC ) REFERENCES "Sales"."Customer" ( "CustomerID" );
ALTER TABLE "Sales"."SalesOrderHeader" ADD CONSTRAINT "ShipMethod" NOT NULL FOREIGN KEY ( "ShipMethodID" ASC ) REFERENCES "Purchasing"."ShipMethod" ( "ShipMethodID" );

ALTER TABLE "Sales"."SalesOrderDetail" ADD CONSTRAINT "Product" NOT NULL FOREIGN KEY ( "ProductID" ASC ) REFERENCES "Production"."Product" ( "ProductID" );
ALTER TABLE "Sales"."SalesOrderDetail" ADD CONSTRAINT "SalesOrderHeader" NOT NULL FOREIGN KEY ( "SalesOrderID" ASC ) REFERENCES "Sales"."SalesOrderHeader" ( "SalesOrderID" ) ON DELETE CASCADE;
ALTER TABLE "Sales"."SalesOrderDetail" ADD CONSTRAINT "SpecialOffer" NOT NULL FOREIGN KEY ( "SpecialOfferID" ASC ) REFERENCES "Sales"."SpecialOffer" ( "SpecialOfferID" );
ALTER TABLE "Sales"."SalesOrderDetail" ADD CONSTRAINT "SpecialOfferProduct" NOT NULL FOREIGN KEY ( "SpecialOfferID" ASC, "ProductID" ASC ) REFERENCES "Sales"."SpecialOfferProduct" ( "SpecialOfferID", "ProductID" );

ALTER TABLE "Sales"."SpecialOfferProduct" ADD CONSTRAINT "SpecialOffer" NOT NULL FOREIGN KEY ( "SpecialOfferID" ASC ) REFERENCES "Sales"."SpecialOffer" ( "SpecialOfferID" );
ALTER TABLE "Sales"."SpecialOfferProduct" ADD CONSTRAINT "Product" NOT NULL FOREIGN KEY ( "ProductID" ASC ) REFERENCES "Production"."Product" ( "ProductID" )

ALTER TABLE "Sales"."Store" ADD CONSTRAINT "Customer" NOT NULL FOREIGN KEY ( "CustomerID" ASC ) REFERENCES "Sales"."Customer" ( "CustomerID" );

ALTER TABLE "Sales"."StoreContact" ADD CONSTRAINT "Contact" NOT NULL FOREIGN KEY ( "ContactID" ASC ) REFERENCES "Person"."Contact" ( "ContactID" );
ALTER TABLE "Sales"."StoreContact" ADD CONSTRAINT "ContactType" NOT NULL FOREIGN KEY ( "ContactTypeID" ASC ) REFERENCES "Person"."ContactType" ( "ContactTypeID" );
ALTER TABLE "Sales"."StoreContact" ADD CONSTRAINT "Customer" NOT NULL FOREIGN KEY ( "CustomerID" ASC ) REFERENCES "Sales"."Customer" ( "CustomerID" );

/*------------------------------------------------------------------------------
*                       Create trigger 'iduSalesOrderDetail'.
*-----------------------------------------------------------------------------*/

CREATE TRIGGER "iduSalesOrderDetail" AFTER INSERT, DELETE, UPDATE
ORDER 1 ON "Sales"."SalesOrderDetail"
REFERENCING NEW AS current_row
FOR EACH ROW
BEGIN
	DECLARE _SubTotal DECIMAL;

	SELECT SUM(Sales.SalesOrderDetail.LineTotal) INTO _SubTotal
		FROM Sales.SalesOrderDetail
		WHERE Sales.SalesOrderDetail.SalesOrderID = current_row.SalesOrderID;

	IF _SubTotal IS NOT NULL THEN
    		UPDATE Sales.SalesOrderHeader
    			SET Sales.SalesOrderHeader.SubTotal = _SubTotal
    			WHERE Sales.SalesOrderHeader.SalesOrderID = current_row.SalesOrderID;

	ELSE
        	UPDATE Sales.SalesOrderHeader
    			SET Sales.SalesOrderHeader.SubTotal = 0
    			WHERE Sales.SalesOrderHeader.SalesOrderID = current_row.SalesOrderID;
   	END IF;
END;

/*------------------------------------------------------------------------------
*                       Create procedure 'sp_Report'.
*-----------------------------------------------------------------------------*/

CREATE PROCEDURE "SalesPerson"."sp_Report"(IN _NumberOfResults INT DEFAULT 10)
RESULT( CustomerName LONG VARCHAR, TotalSales DECIMAL )

BEGIN

SELECT TOP _NumberOfResults Name, TotalSold FROM
(SELECT Sales.Store.CustomerID, SUM(Sales.SalesOrderHeader.TotalDue) AS TotalSold, RANK () OVER (ORDER BY SUM(Sales.SalesOrderHeader.TotalDue) DESC) AS CustomerRank FROM Sales.SalesOrderHeader, Sales.Store WHERE Sales.Store.CustomerID = Sales.SalesOrderHeader.CustomerID GROUP BY Sales.Store.CustomerID) SubQuery,
Sales.Store WHERE Sales.Store.CustomerID = SubQuery.CustomerID
ORDER BY TotalSold DESC;

END;

/*------------------------------------------------------------------------------
*                       Grant Permissions to 'SalesPerson' Group
*-----------------------------------------------------------------------------*/

GRANT SELECT ON "Person"."Address" TO "SalesPerson";
GRANT SELECT ON "Person"."AddressType" TO "SalesPerson";
GRANT SELECT ON "Person"."Contact" TO "SalesPerson";
GRANT SELECT ON "Person"."ContactType" TO "SalesPerson";
GRANT SELECT ON "Sales"."Customer" TO "SalesPerson";
GRANT SELECT ON "Sales"."CustomerAddress" TO "SalesPerson";
GRANT SELECT ON "Production"."Product" TO "SalesPerson";
GRANT SELECT, INSERT, DELETE, UPDATE ON "Sales"."SalesOrderDetail" TO "SalesPerson";
GRANT SELECT, INSERT, DELETE, UPDATE ON "Sales"."SalesOrderHeader" TO "SalesPerson";
GRANT SELECT ON "Sales"."SalesPerson" TO "SalesPerson";
GRANT SELECT ON "Purchasing"."ShipMethod" TO "SalesPerson";
GRANT SELECT ON "Sales"."SpecialOffer" TO "SalesPerson";
GRANT SELECT ON "Sales"."SpecialOfferProduct" TO "SalesPerson";
GRANT SELECT ON "Person"."StateProvince" TO "SalesPerson";
GRANT SELECT ON "Sales"."Store" TO "SalesPerson";
GRANT SELECT ON "Sales"."StoreContact" TO "SalesPerson";

GRANT EXECUTE ON "SalesPerson"."sp_Report" TO "SalesPerson";

/*------------------------------------------------------------------------------
*                       Create publication 'publication'.
*-----------------------------------------------------------------------------*/

CREATE PUBLICATION "publication" (
		TABLE "Person"."Address",
		TABLE "Person"."AddressType",
		TABLE "Person"."Contact",
		TABLE "Person"."ContactType",
		TABLE "Sales"."Customer",
		TABLE "Sales"."CustomerAddress",
		TABLE "Production"."Product",
		TABLE "Sales"."SalesOrderDetail" ("SalesOrderID", "SalesOrderDetailID", "CarrierTrackingNumber", "OrderQty", "ProductID", "SpecialOfferID", "UnitPrice", "UnitPriceDiscount", "rowguid", "ModifiedDate"),
		TABLE "Sales"."SalesOrderHeader" ("SalesOrderID", "RevisionNumber", "OrderDate", "DueDate", "ShipDate", "Status", "OnlineOrderFlag", "PurchaseOrderNumber", "AccountNumber", "CustomerID", "ContactID", "TerritoryID", "BillToAddressID", "ShipToAddressID", "ShipMethodID", "CreditCardID", "CreditCardApprovalCode", "CurrencyRateID", "SubTotal", "TaxAmt", "Freight", "Comment", "rowguid", "ModifiedDate"),
		TABLE "Sales"."SalesPerson",
		TABLE "Purchasing"."ShipMethod",
		TABLE "Sales"."SpecialOffer",
		TABLE "Sales"."SpecialOfferProduct",
		TABLE "Person"."StateProvince",
		TABLE "Sales"."Store",
		TABLE "Sales"."StoreContact"
);

COMMIT;