using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.SqlClient;

namespace AdventureWorksDemo_Consolidated
{
    public partial class AddOrder : Form
    {
        private bool inserted;
        private int orderid;
        private int customerid;
        private int[] salespersonid;
        private SqlTransaction transaction;
        private SqlDataReader backgroundreader;
        private System.Threading.EventWaitHandle signal;

        public AddOrder()
        {
            InitializeComponent();
            signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.AutoReset);
            inserted = false;
            salespersonid = new int[0];
        }

        private void AddOrder_Load(object sender, EventArgs e)
        {
            SqlCommand cmd;
            SqlDataReader reader;
            string sqlstmt;
            orderid = 0;

            transaction = Program.db.BeginTransaction();
            Program.transaction = transaction;

            sqlstmt = "SELECT SalesPersonID, FirstName + ' ' + LastName FROM Person.Contact, HumanResources.Employee, Sales.SalesPerson WHERE Person.Contact.ContactID = HumanResources.Employee.ContactID AND HumanResources.Employee.EmployeeID = Sales.SalesPerson.SalesPersonID";
            cmd = Program.CreateCommand(sqlstmt);
            reader = cmd.ExecuteReader();

            while (reader.Read())
            {
                salesrep.Items.Add(reader.GetString(1));
                Array.Resize<int>(ref salespersonid, salespersonid.Length + 1);
                salespersonid[salespersonid.Length - 1] = reader.GetInt32(0);
            }

            reader.Close();
            reader.Dispose();
            cmd.Dispose();

            status.SelectedIndex = 1;
            salesrep.SelectedIndex = 0;
            orderdate.Value = DateTime.Today;
            duedate.Value = DateTime.Today.AddDays(7);
        }

        private void AddOrder_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (transaction != null)
            {
                transaction.Rollback();
                transaction.Dispose();
                transaction = null;
                Program.transaction = null;
            }
        }

        private void status_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void freight_TextChanged(object sender, EventArgs e)
        {
            try
            {
                totaldue.Text = ((decimal)(System.Convert.ToDecimal(subtotal.Text.Substring(1)) + System.Convert.ToDecimal(freight.Text) + System.Convert.ToDecimal(taxamt.Text))).ToString();
            }
            catch
            {
            }
        }

        private void taxamt_TextChanged(object sender, EventArgs e)
        {
            try
            {
                totaldue.Text = ((decimal)(System.Convert.ToDecimal(subtotal.Text.Substring(1)) + System.Convert.ToDecimal(freight.Text) + System.Convert.ToDecimal(taxamt.Text))).ToString();
            }
            catch
            {
            }
        }

        private void orderdate_ValueChanged(object sender, EventArgs e)
        {
            if (orderdate.Value > duedate.Value)
            {
                duedate.Value = orderdate.Value;
            }
            if (orderdate.Value > shipdate.Value)
            {
                shipdate.Value = orderdate.Value;
            }
        }

        private void duedate_ValueChanged(object sender, EventArgs e)
        {
            if (duedate.Value < orderdate.Value)
            {
                orderdate.Value = duedate.Value;
            }
        }

        private void shipdate_ValueChanged(object sender, EventArgs e)
        {
            if (shipdate.Value < orderdate.Value)
            {
                orderdate.Value = shipdate.Value;
            }
        }

        private void ship_CheckedChanged(object sender, EventArgs e)
        {
            if (ship.Checked)
            {
                shipdate.Enabled = true;
                shipdate.Value = DateTime.Today;
            }
            else
            {
                shipdate.Enabled = false;
            }
        }

        private void comment_TextChanged(object sender, EventArgs e)
        {

        }

        private void onlineorderflag_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void itemlist_SelectedIndexChanged(object sender, EventArgs e)
        {
            edititem.Enabled = true;
            deleteitem.Enabled = true;
        }

        private void findcustomer_Click(object sender, EventArgs e)
        {
            ViewCustomers pickcustomersform = new ViewCustomers();

            pickcustomersform.dialog = true;
            if (pickcustomersform.ShowDialog() == DialogResult.Yes)
            {
                SqlCommand cmd;
                SqlDataReader reader;
                string sqlstmt;
                int contactid;
                int billtoaddressid;
                int shiptoaddressid;
                int shipmethodid;
                
                customerid = (int)pickcustomersform.customerlist.SelectedItems[0].Tag;

                sqlstmt = "SELECT TOP 1 Sales.StoreContact.ContactID, Name FROM Sales.Store, Sales.StoreContact WHERE Sales.StoreContact.CustomerID = Sales.Store.CustomerID AND Sales.Store.CustomerID=" + customerid.ToString();
                cmd = Program.CreateCommand(sqlstmt);
                reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    contactid = reader.GetInt32(0);
                    customername.Text = reader.GetString(1);
                }
                else
                {
                    reader.Close();
                    reader.Dispose();
                    cmd.Dispose();
                    return;
                }
                reader.Close();
                reader.Dispose();
                cmd.Dispose();

                if (!inserted)
                {
                    inserted = true;

                    sqlstmt = "SELECT TOP 1 AddressID FROM Sales.CustomerAddress WHERE Sales.CustomerAddress.CustomerID=" + customerid.ToString();
                    cmd = Program.CreateCommand(sqlstmt);
                    billtoaddressid = (int)cmd.ExecuteScalar();
                    shiptoaddressid = billtoaddressid;
                    cmd.Dispose();

                    sqlstmt = "SELECT TOP 1 ShipMethodID FROM Purchasing.ShipMethod";
                    cmd = Program.CreateCommand(sqlstmt);
                    shipmethodid = (int)cmd.ExecuteScalar();
                    cmd.Dispose();

                    sqlstmt = "INSERT INTO Sales.SalesOrderHeader (";
                    sqlstmt += "RevisionNumber, OrderDate, DueDate, ShipDate, Status, OnlineOrderFlag, CustomerID, ContactID, BillToAddressID, ShipToAddressID, ShipMethodID, SubTotal, Freight, TaxAmt, \"Comment\", SalesPersonID";
                    sqlstmt += ") VALUES (";
                    sqlstmt += "1, ";
                    sqlstmt += "'" + orderdate.Value.ToString("yyyy-MM-dd") + "', ";
                    sqlstmt += "'" + duedate.Value.ToString("yyyy-MM-dd") + "', ";
                    if (ship.Checked)
                    {
                        sqlstmt += "'" + shipdate.Value.ToString("yyyy-MM-dd") + "', ";
                    }
                    else
                    {
                        sqlstmt += "NULL, ";
                    }
                    sqlstmt += status.SelectedIndex.ToString() + ", ";
                    if (onlineorderflag.Checked)
                    {
                        sqlstmt += "1, ";
                    }
                    else
                    {
                        sqlstmt += "0, ";
                    }
                    sqlstmt += customerid.ToString() + ", ";
                    sqlstmt += contactid.ToString() + ", ";
                    sqlstmt += billtoaddressid.ToString() + ", ";
                    sqlstmt += shiptoaddressid.ToString() + ", ";
                    sqlstmt += shipmethodid.ToString() + ", ";
                    sqlstmt += "0, 0, 0, ";
                    sqlstmt += "'" + comment.Text.Replace("\r\n", "\\n") + "', ";
                    sqlstmt += salespersonid[salesrep.SelectedIndex].ToString();
                    sqlstmt += ")";

                    cmd = new SqlCommand(sqlstmt, Program.db, transaction);
                    cmd.ExecuteNonQuery();
                    cmd.Dispose();

                    sqlstmt = "SELECT TOP 1 SalesOrderID FROM Sales.SalesOrderHeader ORDER BY ModifiedDate DESC";
                    cmd = Program.CreateCommand(sqlstmt);
                    orderid = (int)cmd.ExecuteScalar();
                    cmd.Dispose();

                    editcontactaddress.Enabled = true;
                    itemlist.Enabled = true;
                    additem.Enabled = true;
                }
                else
                {
                    sqlstmt = "SELECT TOP 1 AddressID FROM Sales.CustomerAddress WHERE Sales.CustomerAddress.CustomerID=" + customerid.ToString();
                    cmd = Program.CreateCommand(sqlstmt);
                    billtoaddressid = (int)cmd.ExecuteScalar();
                    shiptoaddressid = billtoaddressid;
                    cmd.Dispose();

                    sqlstmt = "SELECT TOP 1 ShipMethodID FROM Purchasing.ShipMethod";
                    cmd = Program.CreateCommand(sqlstmt);
                    shipmethodid = (int)cmd.ExecuteScalar();
                    cmd.Dispose();

                    sqlstmt = "UPDATE Sales.SalesOrderHeader SET ";
                    sqlstmt += "CustomerID=" + customerid.ToString() + ", ";
                    sqlstmt += "ContactID=" + contactid.ToString() + ", ";
                    sqlstmt += "BillToAddressID=" + billtoaddressid.ToString() + ", ";
                    sqlstmt += "ShipToAddressID=" + shiptoaddressid.ToString() + ", ";
                    sqlstmt += "ShipMethodID=" + shipmethodid.ToString() + " ";                   
                    sqlstmt += "WHERE SalesOrderID=" + orderid.ToString();

                    cmd = new SqlCommand(sqlstmt, Program.db, transaction);
                    cmd.ExecuteNonQuery();
                    cmd.Dispose();
                }

                apply.Enabled = true;
                ok.Enabled = true;
            }
        }

        private void editcontactaddress_Click(object sender, EventArgs e)
        {
            EditContactAddress editcontactaddressform = new EditContactAddress();
            editcontactaddressform.orderid = orderid;
            editcontactaddressform.transaction = transaction;
            if (editcontactaddressform.ShowDialog() == DialogResult.OK)
            {
                apply.Enabled = true;
                ok.Enabled = true;
            }
        }

        private void additem_Click(object sender, EventArgs e)
        {
            AddItem additemform = new AddItem();

            additemform.orderid = orderid;
            additemform.transaction = transaction;
            additemform.ShowDialog();

            edititem.Enabled = false;
            deleteitem.Enabled = false;
            this.Enabled = false;
            itemlist.Items.Clear();
            backgroundprocessor.RunWorkerAsync();
        }

        private void edititem_Click(object sender, EventArgs e)
        {
            EditItem edititemform = new EditItem();

            edititemform.orderid = orderid;
            edititemform.orderdetailid = (int)itemlist.SelectedItems[0].Tag;
            edititemform.transaction = transaction;
            edititemform.ShowDialog();

            edititem.Enabled = false;
            deleteitem.Enabled = false;
            this.Enabled = false;
            itemlist.Items.Clear();
            backgroundprocessor.RunWorkerAsync();
        }

        private void deleteitem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete this item?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                SqlCommand cmd;
                string sqlstmt;

                sqlstmt = "DELETE FROM Sales.SalesOrderDetail WHERE Sales.SalesOrderDetail.SalesOrderID=" + orderid.ToString() + " AND Sales.SalesOrderDetail.SalesOrderDetailID=" + ((int)itemlist.SelectedItems[0].Tag).ToString();
                cmd = new SqlCommand(sqlstmt, Program.db, transaction);
                cmd.ExecuteNonQuery();
                cmd.Dispose();

                edititem.Enabled = false;
                deleteitem.Enabled = false;

                itemlist.Items.Clear();
                backgroundprocessor.RunWorkerAsync();
            }
        }

        private void cancel_Click(object sender, EventArgs e)
        {
            transaction.Rollback();
            transaction.Dispose();
            Program.transaction = null;
            transaction = null;
            this.Close();
        }

        private void ok_Click(object sender, EventArgs e)
        {
            apply_Click(null, null);
            this.Close();
        }

        private void apply_Click(object sender, EventArgs e)
        {
            SqlCommand cmd;
            string sqlstmt;

            sqlstmt = "UPDATE Sales.SalesOrderHeader ";
            sqlstmt += "SET ";
            sqlstmt += "OrderDate='" + orderdate.Value.ToString("yyyy-MM-dd") + "', ";
            sqlstmt += "DueDate='" + duedate.Value.ToString("yyyy-MM-dd") + "', ";
            if (ship.Checked)
            {
                sqlstmt += "ShipDate='" + shipdate.Value.ToString("yyyy-MM-dd") + "', ";
            }
            else
            {
                sqlstmt += "ShipDate=NULL, ";
            }
            if (onlineorderflag.Checked)
            {
                sqlstmt += "OnlineOrderFlag=1, ";
            }
            else
            {
                sqlstmt += "OnlineOrderFlag=0, ";
            }
            sqlstmt += "CustomerID=" + customerid.ToString() + ", ";
            sqlstmt += "Freight=" + freight.Text + ", ";
            sqlstmt += "TaxAmt=" + taxamt.Text + ", ";
            sqlstmt += "\"Comment\"='" + comment.Text.Replace("\r\n","\\n") + "', ";
            sqlstmt += "SalesPersonID=" + salespersonid[salesrep.SelectedIndex].ToString() + " ";
            sqlstmt += "WHERE SalesOrderID=" + orderid.ToString();

            cmd = new SqlCommand(sqlstmt, Program.db, transaction);
            cmd.ExecuteNonQuery();
            cmd.Dispose();

            transaction.Commit();
            transaction.Dispose();
            transaction = Program.db.BeginTransaction();
            Program.transaction = transaction;
        }

        private void backgroundprocess(object sender, DoWorkEventArgs e)
        {
            SqlCommand backgroundcmd;
            string sqlstmt;

            sqlstmt = "SELECT SalesOrderDetailID, ProductNumber, Name, OrderQty, UnitPrice, UnitPriceDiscount, LineTotal, Description, Sales.SalesOrderDetail.ProductID FROM Sales.SalesOrderDetail, Production.Product, Sales.SpecialOffer WHERE Sales.SpecialOffer.SpecialOfferID = Sales.SalesOrderDetail.SpecialOfferID AND Production.Product.ProductID = Sales.SalesOrderDetail.ProductID AND Sales.SalesOrderDetail.SalesOrderID=" + orderid.ToString();
            backgroundcmd = Program.CreateCommand(sqlstmt);
            backgroundreader = backgroundcmd.ExecuteReader();
            
            while (backgroundreader.Read())
            {
                (sender as BackgroundWorker).ReportProgress(1);
                signal.WaitOne(1000, false);
            }

            backgroundreader.Close();
            backgroundreader.Dispose();
            backgroundcmd.Dispose();
        }

        private void backgroundprogressupdate(object sender, ProgressChangedEventArgs e)
        {
            int itemindex = itemlist.Items.Count;

            itemlist.Items.Add(backgroundreader.GetString(1));
            itemlist.Items[itemindex].Tag = backgroundreader.GetInt32(0);
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetString(2));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetInt16(3).ToString());
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(4).ToString("$###,##0.00"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(5).ToString("0.00#%"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(6).ToString("$###,##0.00"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetString(7));
            itemlist.Items[itemindex].SubItems[0].Tag = backgroundreader.GetInt32(8);
            signal.Set();
        }

        private void backgroundcompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            SqlCommand cmd;
            string sqlstmt;

            sqlstmt = "SELECT SubTotal FROM Sales.SalesOrderHeader WHERE SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            subtotal.Text = ((decimal)cmd.ExecuteScalar()).ToString("$###,##0.00");
            cmd.Dispose();
            sqlstmt = "SELECT TotalDue FROM Sales.SalesOrderHeader WHERE SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            totaldue.Text = ((decimal)cmd.ExecuteScalar()).ToString("$###,##0.00");

            cmd.Dispose();

            this.Enabled = true;
        }
    }
}