using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.SqlClient;

namespace AdventureWorksDemo_Consolidated
{
    public partial class ViewOrderDetails : Form
    {
        public int orderid
        {
            private get
            {
                return _orderid;
            }
            set
            {
                _orderid = value;
            }
        }
        public bool hasbeenedited
        {
            get
            {
                return _hasbeenedited;
            }
            private set
            {
                _hasbeenedited = value;
            }
        }
        private int _orderid;
        private bool _hasbeenedited;
        private SqlDataReader backgroundreader;
        private int itemcount = 0;
        private System.Threading.EventWaitHandle signal;

        public ViewOrderDetails()
        {
            InitializeComponent();
            signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.AutoReset);
            hasbeenedited = false;
        }

        private void ViewOrderDetails_Load(object sender, EventArgs e)
        {
            SqlCommand cmd;
            SqlDataReader reader;
            string sqlstmt;

            sqlstmt = "SELECT SalesOrderNumber, RevisionNumber, Status, OrderDate, DueDate, ShipDate, SubTotal, Freight, TaxAmt, TotalDue, FirstName + ' ' + LastName, \"Comment\", OnlineOrderFlag, Sales.SalesOrderHeader.CustomerID, Sales.Store.Name, Sales.SalesOrderHeader.ModifiedDate FROM Sales.SalesOrderHeader, Sales.Store, Person.Contact, HumanResources.Employee WHERE Person.Contact.ContactID = HumanResources.Employee.ContactID AND HumanResources.Employee.EmployeeID = Sales.SalesOrderHeader.SalesPersonID AND Sales.Store.CustomerID = Sales.SalesOrderHeader.CustomerID AND SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            reader = cmd.ExecuteReader();

            if (reader.Read())
            {
                int customerid;

                saleordernumber.Text = reader.GetString(0);
                revisionnumber.Text = reader.GetByte(1).ToString();
                switch (reader.GetByte(2))
                {
                    case 1:
                        status.Text = "In Process";
                        break;
                    case 2:
                        status.Text = "Approved";
                        break;
                    case 3:
                        status.Text = "Backordered";
                        break;
                    case 4:
                        status.Text = "Rejected";
                        break;
                    case 5:
                        status.Text = "Shipped";
                        break;
                    case 6:
                        status.Text = "Cancelled";
                        break;
                    default:
                        status.Text = "Not Available";
                        break;
                }
                orderdate.Text = reader.GetDateTime(3).ToString("ddd, MMM dd, yyyy");
                duedate.Text = reader.GetDateTime(4).ToString("ddd, MMM dd, yyyy");
                if (!reader.IsDBNull(5))
                {
                    shipdate.Text = reader.GetDateTime(5).ToString("ddd, MMM dd, yyyy");
                }
                subtotal.Text = reader.GetDecimal(6).ToString("$###,##0.00");
                freight.Text = reader.GetDecimal(7).ToString("$###,##0.00");
                taxamt.Text = reader.GetDecimal(8).ToString("$###,##0.00");
                totaldue.Text = reader.GetDecimal(9).ToString("$###,##0.00");
                salesrep.Text = reader.GetString(10);
                if (!reader.IsDBNull(11))
                {
                    comment.Text = reader.GetString(11).Replace("\\n", "\r\n");
                }
                onlineorderflag.Checked = reader.GetBoolean(12);
                customerid = reader.GetInt32(13);
                customername.Text = reader.GetString(14);
                modifieddate.Text = reader.GetDateTime(15).ToString();
            }

            reader.Close();
            reader.Dispose();
            cmd.Dispose();

            backgroundprocessor.RunWorkerAsync();
        }

        private void edit_Click(object sender, EventArgs e)
        {
            EditOrderDetails editorderdetailsform = new EditOrderDetails();
            editorderdetailsform.orderid = orderid;
            editorderdetailsform.ShowDialog();

            hasbeenedited = true;

            this.Close();
        }


        private void delete_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete this order?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
            {
                SqlCommand cmd;
                string sqlstmt;

                sqlstmt = "DELETE FROM Sales.SalesOrderHeader WHERE SalesOrderID=" + orderid.ToString();
                cmd = Program.CreateCommand(sqlstmt);
                cmd.ExecuteNonQuery();

                sqlstmt = "DELETE FROM Sales.SalesOrderDetail WHERE SalesOrderID=" + orderid.ToString();
                cmd = Program.CreateCommand(sqlstmt);
                cmd.ExecuteNonQuery();

                hasbeenedited = true;

                this.Close();
            }
        }

        private void viewcontactaddress_Click(object sender, EventArgs e)
        {
            ViewContactAddress viewcontactaddressform = new ViewContactAddress();
            viewcontactaddressform.orderid = orderid;
            viewcontactaddressform.ShowDialog();
        }

        private void itemlist_ItemActivate(object sender, EventArgs e)
        {
            ViewProductDetails viewproductdetailsform = new ViewProductDetails();
            viewproductdetailsform.productid = (int)itemlist.SelectedItems[0].Tag;
            viewproductdetailsform.Show();
        }

        private void ok_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void backgroundprocess(object sender, DoWorkEventArgs e)
        {
            SqlCommand backgroundcmd;
            string sqlstmt;
            

            sqlstmt = "SELECT ProductNumber, Name, OrderQty, UnitPrice, UnitPriceDiscount, LineTotal, Description, Sales.SalesOrderDetail.ProductID FROM Sales.SalesOrderDetail, Production.Product, Sales.SpecialOffer WHERE Sales.SpecialOffer.SpecialOfferID = Sales.SalesOrderDetail.SpecialOfferID AND Production.Product.ProductID = Sales.SalesOrderDetail.ProductID AND Sales.SalesOrderDetail.SalesOrderID=" + orderid.ToString();
            backgroundcmd = Program.CreateCommand(sqlstmt);
            backgroundreader = backgroundcmd.ExecuteReader();
            
            while (backgroundreader.Read())
            {
                (sender as BackgroundWorker).ReportProgress(1);
                signal.WaitOne(1000, false);
            }

            backgroundreader.Close();
            backgroundreader.Dispose();
            backgroundcmd.Dispose();
        }

        private void backgroundprogressupdate(object sender, ProgressChangedEventArgs e)
        {
            itemlist.Items.Add(backgroundreader.GetString(0));
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetString(1));
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetInt16(2).ToString());
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetDecimal(3).ToString("$###,##0.00"));
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetDecimal(4).ToString("0.00#%"));
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetDecimal(5).ToString("$###,##0.00"));
            itemlist.Items[itemcount].SubItems.Add(backgroundreader.GetString(6));
            itemlist.Items[itemcount++].Tag = backgroundreader.GetInt32(7);
            signal.Set();
        }

        private void backgroundcompleted(object sender, RunWorkerCompletedEventArgs e)
        {

        }
    }
}