using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Data.SqlClient;
using AdventureWorksDemo_Consolidated.Properties;

namespace AdventureWorksDemo_Consolidated
{
    public partial class ViewOrders : Form
    {
        public bool isedit
        {
            private get
            {
                return _isedit;
            }
            set
            {
                _isedit = value;
            }
        }
        private bool _isedit;
        private SqlDataReader backgroundreader;
        private System.Threading.EventWaitHandle signal;
        private int ordercount;

        public ViewOrders()
        {
            InitializeComponent();
            isedit = false;
            signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.AutoReset);
        }

        private void ViewOrders_Shown(object sender, EventArgs e)
        {
            search_Click(sender, e);
        }

        private void ViewOrders_Deactivate(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void orderlist_ItemActivate(object sender, EventArgs e)
        {
            if (isedit)
            {
                EditOrderDetails editdetailsform = new EditOrderDetails();
                editdetailsform.orderid = (int)orderlist.SelectedItems[0].Tag;
                editdetailsform.ShowDialog();

                orderlist.Items.Clear();
                search_Click(sender, e);
            }
            else
            {
                ViewOrderDetails viewdetailsform = new ViewOrderDetails();
                viewdetailsform.orderid = (int)orderlist.SelectedItems[0].Tag;
                viewdetailsform.ShowDialog();

                if (viewdetailsform.hasbeenedited)
                {
                    orderlist.Items.Clear();
                    search_Click(sender, e);
                }
            }
        }

        private void search_useorderdate_CheckedChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
            if (search_useorderdate.Checked)
            {
                search_orderdate.Enabled = true;
            }
            else
            {
                search_orderdate.Enabled = false;
            }
        }

        private void search_useduedate_CheckedChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
            if (search_useduedate.Checked)
            {
                search_duedate.Enabled = true;
            }
            else
            {
                search_duedate.Enabled = false;
            }
        }

        private void search_status_SelectedIndexChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_status_Click(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_orderdate_ValueChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_duedate_ValueChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_duedate_DropDown(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_orderdate_DropDown(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        public void search_Click(object sender, EventArgs e)
        {
            string sqlstmt;
            string whereclause = "";

            sqlstmt = "SELECT TOP 50 SalesOrderNumber, FirstName + ' ' + LastName, OrderDate, DueDate, Status, TotalDue, SalesOrderID FROM Sales.SalesOrderHeader, Person.Contact, HumanResources.Employee WHERE Person.Contact.ContactID = HumanResources.Employee.ContactID AND HumanResources.Employee.EmployeeID = Sales.SalesOrderHeader.SalesPersonID";
            if (search_ordernumber.Text.Length > 0)
            {
                whereclause += " AND SalesOrderNumber LIKE '%" + search_ordernumber.Text + "%'";
            }
            if (search_useorderdate.Checked)
            {
                whereclause += " AND OrderDate='" + search_orderdate.Value.ToString("yyyy-MM-dd") + "'";
            }
            if (search_useduedate.Checked)
            {
                whereclause += " AND DueDate='" + search_duedate.Value.ToString("yyyy-MM-dd") + "'";
            }
            if (search_status.SelectedIndex > 0)
            {
                whereclause += " AND Status=" + search_status.SelectedIndex.ToString();
            }
            if (whereclause.Length > 5)
            {
                sqlstmt += " WHERE " + whereclause.Substring(5);
            }
            sqlstmt += " ORDER BY OrderDate DESC, DueDate ASC";

            search.Enabled = false;
            stop.Enabled = true;
            ordercount = 0;
            orderlist.Items.Clear();
            orderlist.Enabled = false;
            orderlist.Visible = false;

            (MdiParent as MainForm).statusbarlabel.Text = "Searching for orders...";
            (MdiParent as MainForm).statusbarprogress.Value = (MdiParent as MainForm).statusbarprogress.Maximum;

            backgroundprocessor.RunWorkerAsync(sqlstmt);
        }

        private void stop_Click(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_ordernumber_TextChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void backgroundprocess(object sender, DoWorkEventArgs e)
        {
            SqlCommand backgroundcmd;
            string sqlstmt = e.Argument as string;
            bool done = false;

            backgroundcmd = Program.CreateCommand(sqlstmt);
            backgroundreader = backgroundcmd.ExecuteReader();
            
            while (!((sender as BackgroundWorker).CancellationPending) && !done)
            {
                if (backgroundreader.Read())
                {
                    (sender as BackgroundWorker).ReportProgress(1);
                    signal.WaitOne(1000,false);
                }
                else
                {
                    done = true;
                }
            }

            backgroundreader.Close();
            backgroundreader.Dispose();
            backgroundcmd.Dispose();
        }

        private void backgroundprogressupdate(object sender, ProgressChangedEventArgs e)
        {
            orderlist.Items.Add(backgroundreader.GetString(0));
            orderlist.Items[ordercount].SubItems.Add(backgroundreader.GetString(1));
            orderlist.Items[ordercount].SubItems.Add(backgroundreader.GetDateTime(2).ToString("ddd, MMM dd, yyyy"));
            orderlist.Items[ordercount].SubItems.Add(backgroundreader.GetDateTime(3).ToString("ddd, MMM dd, yyyy"));
            orderlist.Items[ordercount].SubItems.Add(search_status.Items[backgroundreader.GetByte(4)].ToString());
            orderlist.Items[ordercount].SubItems.Add(backgroundreader.GetDecimal(5).ToString("$###,##0.00"));
            orderlist.Items[ordercount++].Tag = backgroundreader.GetInt32(6);
            (MdiParent as MainForm).statusbarlabel.Text = "Searching for orders... Found " + ordercount.ToString() + " orders.";
            signal.Set();
        }

        private void backgroundcompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            orderlist.Enabled = true;
            orderlist.Visible = true;
            stop.Enabled = false;
            search.Enabled = true;

            (MdiParent as MainForm).statusbarlabel.Text = "Finished. Found " + ordercount.ToString() + " orders.";
            (MdiParent as MainForm).statusbarprogress.Value = (MdiParent as MainForm).statusbarprogress.Minimum;
        }
    }
}