using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using iAnywhere.Data.SQLAnywhere;

namespace AdventureWorksDemo_Remote
{
    public partial class EditOrderDetails : Form
    {
        public int orderid
        {
            private get
            {
                return _orderid;
            }
            set
            {
                _orderid = value;
            }
        }
        private int _orderid;
        private SATransaction transaction;
        private int customerid;
        private SADataReader backgroundreader;
        private System.Threading.EventWaitHandle signal;

        public EditOrderDetails()
        {
            InitializeComponent();
            signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.AutoReset);
        }

        private void EditOrderDetails_Load(object sender, EventArgs e)
        {
            SACommand cmd;
            SADataReader reader;
            string sqlstmt;

            transaction = Program.db.BeginTransaction();
            Program.transaction = transaction;

            sqlstmt = "SELECT SalesOrderNumber, RevisionNumber, Status, OrderDate, DueDate, ShipDate, SubTotal, Freight, TaxAmt, TotalDue, \"Comment\", OnlineOrderFlag, Sales.Store.CustomerID, Sales.Store.Name, Sales.SalesOrderHeader.ModifiedDate FROM Sales.SalesOrderHeader, Sales.Store WHERE Sales.Store.CustomerID = Sales.SalesOrderHeader.CustomerID AND SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            reader = cmd.ExecuteReader();

            if (reader.Read())
            {
                saleordernumber.Text = reader.GetString(0);
                status.SelectedIndex = reader.GetByte(2);
                orderdate.Value = reader.GetDateTime(3);
                duedate.Value = reader.GetDateTime(4);

                if (!reader.IsDBNull(5))
                {
                    ship.Checked = true;
                    shipdate.Value = reader.GetDateTime(5);
                }
                subtotal.Text = reader.GetDecimal(6).ToString("$###,##0.00");
                freight.Text = reader.GetDecimal(7).ToString("0.00");
                taxamt.Text = reader.GetDecimal(8).ToString("0.00");
                totaldue.Text = reader.GetDecimal(9).ToString("$###,##0.00");
                if (!reader.IsDBNull(10))
                {
                    comment.Text = reader.GetString(10).Replace("\\n","\\r\\n");
                }
                onlineorderflag.Checked = reader.GetBoolean(11);
                customerid = reader.GetInt32(12);
                customername.Text = reader.GetString(13);
            }

            reader.Close();
            reader.Dispose();
            cmd.Dispose();

            this.Enabled = false;
            backgroundprocessor.RunWorkerAsync();
        }

        private void EditOrderDetails_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (transaction != null)
            {
                transaction.Rollback();
                transaction.Dispose();
                transaction = null;
                Program.transaction = null;
            }
        }

        private void ship_CheckedChanged(object sender, EventArgs e)
        {
            if (ship.Checked)
            {
                shipdate.Enabled = true;
                shipdate.Value = DateTime.Today;
            }
            else
            {
                shipdate.Enabled = false;
            }
        }

        private void status_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void freight_TextChanged(object sender, EventArgs e)
        {
            try
            {
                totaldue.Text = ((decimal)(System.Convert.ToDecimal(subtotal.Text.Substring(1)) + System.Convert.ToDecimal(freight.Text) + System.Convert.ToDecimal(taxamt.Text))).ToString("$###,##0.00");
            }
            catch
            {
            }
        }

        private void taxamt_TextChanged(object sender, EventArgs e)
        {
            try
            {
                totaldue.Text = ((decimal)(System.Convert.ToDecimal(subtotal.Text.Substring(1)) + System.Convert.ToDecimal(freight.Text) + System.Convert.ToDecimal(taxamt.Text))).ToString("$###,##0.00");
            }
            catch
            {
            }
        }

        private void orderdate_ValueChanged(object sender, EventArgs e)
        {
            if (orderdate.Value > duedate.Value)
            {
                duedate.Value = orderdate.Value;
            }
            if (orderdate.Value > shipdate.Value)
            {
                shipdate.Value = orderdate.Value;
            }
        }

        private void duedate_ValueChanged(object sender, EventArgs e)
        {
            if (duedate.Value < orderdate.Value)
            {
                orderdate.Value = duedate.Value;
            }
        }

        private void shipdate_ValueChanged(object sender, EventArgs e)
        {
            if (shipdate.Value < orderdate.Value)
            {
                orderdate.Value = shipdate.Value;
            }
        }

        private void comment_TextChanged(object sender, EventArgs e)
        {

        }

        private void onlineorderflag_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void itemlist_SelectedIndexChanged(object sender, EventArgs e)
        {
            edititem.Enabled = true;
            deleteitem.Enabled = true;
        }

        private void itemlist_ItemActivate(object sender, EventArgs e)
        {
            ViewProductDetails viewproductdetailsform = new ViewProductDetails();
            viewproductdetailsform.productid = (int)itemlist.SelectedItems[0].SubItems[0].Tag;
            viewproductdetailsform.Show();
        }

        private void editcontactaddress_Click(object sender, EventArgs e)
        {
            EditContactAddress editcontactaddressform = new EditContactAddress();
            editcontactaddressform.orderid = orderid;
            editcontactaddressform.transaction = transaction;
            if (editcontactaddressform.ShowDialog() == DialogResult.OK)
            {
                apply.Enabled = true;
                ok.Enabled = true;
            }
        }

        private void findcustomer_Click(object sender, EventArgs e)
        {
            ViewCustomers pickcustomersform = new ViewCustomers();

            pickcustomersform.dialog = true;
            if (pickcustomersform.ShowDialog() == DialogResult.Yes)
            {
                SACommand cmd;
                SADataReader reader;
                string sqlstmt;
                int contactid = 0;
                int billtoaddressid;
                int shiptoaddressid;
                int shipmethodid;

                customerid = (int)pickcustomersform.customerlist.SelectedItems[0].Tag;

                sqlstmt = "SELECT Sales.StoreContact.ContactID, Name FROM Sales.Store, Sales.StoreContact WHERE Sales.StoreContact.CustomerID = Sales.Store.CustomerID AND Sales.Store.CustomerID=" + customerid.ToString();
                cmd = Program.CreateCommand(sqlstmt);
                reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    contactid = reader.GetInt32(0);
                    customername.Text = reader.GetString(1);
                }
                else
                {
                    reader.Close();
                    reader.Dispose();
                    cmd.Dispose();
                    return;
                }
                reader.Close();
                reader.Dispose();
                cmd.Dispose();

                sqlstmt = "SELECT TOP 1 AddressID FROM Sales.CustomerAddress WHERE Sales.CustomerAddress.CustomerID=" + customerid.ToString();
                cmd = Program.CreateCommand(sqlstmt);
                billtoaddressid = (int)cmd.ExecuteScalar();
                shiptoaddressid = billtoaddressid;
                cmd.Dispose();

                sqlstmt = "SELECT TOP 1 ShipMethodID FROM Purchasing.ShipMethod";
                cmd = Program.CreateCommand(sqlstmt);
                shipmethodid = (int)cmd.ExecuteScalar();
                cmd.Dispose();

                sqlstmt = "UPDATE Sales.SalesOrderHeader SET ";
                sqlstmt += "CustomerID=" + customerid.ToString() + ", ";
                sqlstmt += "ContactID=" + contactid.ToString() + ", ";
                sqlstmt += "BillToAddressID=" + billtoaddressid.ToString() + ", ";
                sqlstmt += "ShipToAddressID=" + shiptoaddressid.ToString() + ", ";
                sqlstmt += "ShipMethodID=" + shipmethodid.ToString() + " ";
                sqlstmt += "WHERE SalesOrderID=" + orderid.ToString();

                cmd = new SACommand(sqlstmt, Program.db, transaction);
                cmd.ExecuteNonQuery();
                cmd.Dispose();

                apply.Enabled = true;
                ok.Enabled = true;
            }
        }

        private void edititem_Click(object sender, EventArgs e)
        {
            EditItem edititemform = new EditItem();

            edititemform.orderid = orderid;
            edititemform.orderdetailid = (int)itemlist.SelectedItems[0].Tag;
            edititemform.transaction = transaction;
            edititemform.ShowDialog();

            edititem.Enabled = false;
            deleteitem.Enabled = false;
            this.Enabled = false;
            itemlist.Items.Clear();
            backgroundprocessor.RunWorkerAsync();
        }

        private void additem_Click(object sender, EventArgs e)
        {
            AddItem additemform = new AddItem();
            additemform.orderid = orderid;
            additemform.transaction = transaction;
            additemform.ShowDialog();

            edititem.Enabled = false;
            deleteitem.Enabled = false;
            this.Enabled = false;
            itemlist.Items.Clear();
            backgroundprocessor.RunWorkerAsync();
        }

        private void deleteitem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete this item?", "Warning", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                SACommand cmd;
                string sqlstmt;

                sqlstmt = "DELETE FROM Sales.SalesOrderDetail WHERE Sales.SalesOrderDetail.SalesOrderID=" + orderid.ToString() + " AND Sales.SalesOrderDetail.SalesOrderDetailID=" + ((int)itemlist.SelectedItems[0].Tag).ToString();
                cmd = new SACommand(sqlstmt, Program.db, transaction);
                cmd.ExecuteNonQuery();
                cmd.Dispose();

                edititem.Enabled = false;
                deleteitem.Enabled = false;

                itemlist.Items.Clear();

                backgroundprocessor.RunWorkerAsync();
            }
        }

        private void cancel_Click(object sender, EventArgs e)
        {
            transaction.Rollback();
            transaction.Dispose();
            Program.transaction = null;
            transaction = null;
            this.Close();
        }

        private void ok_Click(object sender, EventArgs e)
        {
            apply_Click(null, null);
            this.Close();
        }

        private void apply_Click(object sender, EventArgs e)
        {
            SACommand cmd;
            string sqlstmt;

            sqlstmt = "UPDATE Sales.SalesOrderHeader ";
            sqlstmt += "SET ";
            sqlstmt += "OrderDate='" + orderdate.Value.ToString("yyyy-MM-dd") + "', ";
            sqlstmt += "DueDate='" + duedate.Value.ToString("yyyy-MM-dd") + "', ";
            if (ship.Checked)
            {
                sqlstmt += "ShipDate='" + shipdate.Value.ToString("yyyy-MM-dd") + "', ";
            }
            else
            {
                sqlstmt += "ShipDate=NULL, ";
            }
            sqlstmt += "Status=" + status.SelectedIndex.ToString() + ", ";
            if (onlineorderflag.Checked)
            {
                sqlstmt += "OnlineOrderFlag=1, ";
            }
            else
            {
                sqlstmt += "OnlineOrderFlag=0, ";
            }
            sqlstmt += "CustomerID=" + customerid.ToString() + ", ";
            sqlstmt += "Freight=" + freight.Text + ", ";
            sqlstmt += "TaxAmt=" + taxamt.Text + ", ";
            sqlstmt += "\"Comment\"='" + comment.Text.Replace("\r\n", "\\n") + "' ";
            sqlstmt += "WHERE SalesOrderID=" + orderid.ToString();

            cmd = new SACommand(sqlstmt, Program.db, transaction);
            cmd.ExecuteNonQuery();
            cmd.Dispose();

            transaction.Commit();
            transaction.Dispose();
            transaction = Program.db.BeginTransaction();
            Program.transaction = transaction;
        }       

        private void backgroundprocess(object sender, DoWorkEventArgs e)
        {
            SACommand backgroundcmd;
            string sqlstmt;

            sqlstmt = "SELECT SalesOrderDetailID, ProductNumber, Name, OrderQty, UnitPrice, UnitPriceDiscount, LineTotal, Description, Sales.SalesOrderDetail.ProductID FROM Sales.SalesOrderDetail, Production.Product, Sales.SpecialOffer WHERE Sales.SpecialOffer.SpecialOfferID = Sales.SalesOrderDetail.SpecialOfferID AND Production.Product.ProductID = Sales.SalesOrderDetail.ProductID AND Sales.SalesOrderDetail.SalesOrderID=" + orderid.ToString();
            backgroundcmd = Program.CreateCommand(sqlstmt);
            backgroundreader = backgroundcmd.ExecuteReader();
            
            while (backgroundreader.Read())
            {
                (sender as BackgroundWorker).ReportProgress(1);
                signal.WaitOne(1000, false);
            }

            backgroundreader.Close();
            backgroundreader.Dispose();
            backgroundcmd.Dispose();
        }

        private void backgroundprogressupdate(object sender, ProgressChangedEventArgs e)
        {
            int itemindex = itemlist.Items.Count;

            itemlist.Items.Add(backgroundreader.GetString(1));
            itemlist.Items[itemindex].Tag = backgroundreader.GetInt32(0);
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetString(2));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetInt16(3).ToString());
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(4).ToString("$###,##0.00"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(5).ToString("0.00#%"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetDecimal(6).ToString("$###,##0.00"));
            itemlist.Items[itemindex].SubItems.Add(backgroundreader.GetString(7));
            itemlist.Items[itemindex].SubItems[0].Tag = backgroundreader.GetInt32(8);
            signal.Set();
        }

        private void backgroundcompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            SACommand cmd;
            string sqlstmt;

            sqlstmt = "SELECT SubTotal FROM Sales.SalesOrderHeader WHERE SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            subtotal.Text = ((decimal)cmd.ExecuteScalar()).ToString("$###,##0.00");
            cmd.Dispose();
            sqlstmt = "SELECT TotalDue FROM Sales.SalesOrderHeader WHERE SalesOrderID=" + orderid.ToString();
            cmd = Program.CreateCommand(sqlstmt);
            totaldue.Text = ((decimal)cmd.ExecuteScalar()).ToString("$###,##0.00");

            cmd.Dispose();

            this.Enabled = true;
        }
    }
}