using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using iAnywhere.Data.SQLAnywhere;
using AdventureWorksDemo_Remote.Properties;

namespace AdventureWorksDemo_Remote
{
    public partial class MainForm : Form
    {
        private ViewProducts viewproductsform;
        private ViewCustomers viewcustomersform;
        private ViewOrders viewordersform;
        private Synchronize syncform;
        private DbmlsyncCOM.DbmlsyncClass dbmlsync;
        private bool dbmlsyncerror;

        public MainForm()
        {
            InitializeComponent();
            viewproductsform = new ViewProducts();
            viewcustomersform = new ViewCustomers();
            viewordersform = new ViewOrders();
            dbmlsyncerror = false;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            login_Click(sender, e);
        }

        private void login_Click(object sender, EventArgs e)
        {
            bool tryagain = false;
            /*
             * The Login form is a small dialog, so it can be respawned every call.
             */
            Login loginform = new Login();

            /*
             * Fill in the login credentials automatically with defaults.
             */
            loginform.uid.Text = "jillian0";
            loginform.pwd.Text = "secret";
            loginform.ShowDialog();

            if (loginform.DialogResult == DialogResult.OK)
            {
                string conn;
                Program.uid = loginform.uid.Text;
                Program.pwd = loginform.pwd.Text;
                
                conn = "DBF=" + Settings.Default.dbf;
                conn += ";UID=" + Program.uid;
                conn += ";PWD=" + Program.pwd;

                /*
                 * Try to connect to the db, otherwise ask for retry and show error
                 *  NOTE: If login successful, it is necessary to enable all menu items.
                 */
                try
                {
                    Program.db = new SAConnection(conn);
                    Program.db.Open();

                    ordersmenu.Enabled = true;
                    viewmenu.Enabled = true;
                    logoff.Enabled = true;
                    login.Enabled = false;
                    connect.Enabled = true;

                    vieworders_Click(sender, e);
                }
                catch (SAException exception)
                {
                    statusbar.Items[1].Text = "DATABASE LOGIN FAILED: " + exception.Message;
                    if (MessageBox.Show("DATABASE LOGIN FAILED: " + exception.Message, "Login Error", MessageBoxButtons.RetryCancel, MessageBoxIcon.Error) == DialogResult.Retry)
                    {
                        tryagain = true;
                    }

                    ordersmenu.Enabled = false;
                    viewmenu.Enabled = false;
                    logoff.Enabled = false;
                    login.Enabled = true;
                    connect.Enabled = false;
                }
                catch (NullReferenceException exception)
                {
                    statusbar.Items[1].Text = "DATABASE LOGIN WARNING: Use of unsynchronized remote. " + exception.Message;
                    MessageBox.Show("Warning: The remote database must be manually synchronized at least once before successful login can occur.\r\nOnly the Synchronize menu item will be available.", "Datbase Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    
                    ordersmenu.Enabled = false;
                    viewmenu.Enabled = false;
                    logoff.Enabled = true;
                    login.Enabled = false;
                    connect.Enabled = true;
                }
            }

            if (tryagain)
            {
                login_Click(sender, e);
            }
        }

        private void logoff_Click(object sender, EventArgs e)
        {
            /*
             * Disconnect and dispose the connection.
             *  It is necessary to dispose the connection so the application does not try and
             *  re-disconnect.
             *  NOTE: If the user does NOT logoff, the application will try to disconnect automatically
             *        if it sees that the db object is not null. See Main() function.
             *  NOTE: If login successful, it is necessary to enable all menu items.
             */
            Program.db.Close();
            Program.db.Dispose();

            /*
             * Close all open child windows.
             */
            foreach (Form child in this.MdiChildren)
            {
                child.Close();
            }

            /*
             * Re-disable menu items that require db interactivity.
             */
            ordersmenu.Enabled = false;
            viewmenu.Enabled = false;
            logoff.Enabled = false;
            login.Enabled = true;
            connect.Enabled = false;
        }

        private void connect_Click(object sender, EventArgs e)
        {
            string cmdline;
            dbmlsync = new DbmlsyncCOM.DbmlsyncClass();

            /*
             * The Synch form is a small dialog, so it can be respawned every call.
             */
            syncform = new Synchronize();
            syncform.MdiParent = this;
            syncform.FormClosed += new FormClosedEventHandler(syncform_FormClosed);
            syncform.Show();

            /*
             * Generate the DBMLSync command.
             */
            cmdline = "-c \"DBF=" + Settings.Default.dbf + ";UID=" + Program.uid + ";PWD=" + Program.pwd + "\" ";
            cmdline += "-n \"" + Settings.Default.publication + "\" -u " + Program.uid + " -mp " + Program.pwd;
            cmdline += " -e \"sv=" + Settings.Default.version + ";adr='" + Settings.Default.mlsrvaddress + "'\"";

            /*
             * Implement DBMLSync Event Handles. All event handles located below.
             */
            dbmlsync.Message += new DbmlsyncCOM._IDbmlsyncEvents_MessageEventHandler(dbmlsync_Message);
            dbmlsync.ProgressIndex += new DbmlsyncCOM._IDbmlsyncEvents_ProgressIndexEventHandler(dbmlsync_ProgressIndex);
            dbmlsync.ProgressMessage += new DbmlsyncCOM._IDbmlsyncEvents_ProgressMessageEventHandler(dbmlsync_ProgressMessage);
            dbmlsync.DoneExecution += new DbmlsyncCOM._IDbmlsyncEvents_DoneExecutionEventHandler(dbmlsync_DoneExecution);
            dbmlsync.Run(cmdline);
        }

        private void exit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void enterorder_Click(object sender, EventArgs e)
        {
            /*
             * The AddOrderForm form MUST be shown modally, and hence using ShowDialog.
             *  This is because of transaction handling.
             *  Therefore, it must be spawned new every call.
             */
            AddOrder addorderform = new AddOrder();
            addorderform.ShowDialog();

            if (viewordersform.Visible)
            {
                viewordersform.search_Click(sender, e);
            }
        }

        private void viewproducts_Click(object sender, EventArgs e)
        {
            /*
             * Reuse the same instance of the form everytime
             */
            if (viewproductsform.IsDisposed)
            {
                viewproductsform = new ViewProducts();
            }
            viewproductsform.MdiParent = this;
            viewproductsform.Show();
            viewproductsform.WindowState = FormWindowState.Maximized;
        }

        private void viewcustomers_Click(object sender, EventArgs e)
        {
            /*
             * Reuse the same instance of the form everytime
             */
            if (viewcustomersform.IsDisposed)
            {
                viewcustomersform = new ViewCustomers();
            }
            viewcustomersform.MdiParent = this;
            viewcustomersform.Show();
            viewcustomersform.WindowState = FormWindowState.Maximized;
        }

        private void vieworders_Click(object sender, EventArgs e)
        {
            /*
             * Reuse the same instance of the form everytime
             */
            if (viewordersform.IsDisposed)
            {
                viewordersform = new ViewOrders();
            }
            viewordersform.MdiParent = this;
            viewordersform.isedit = false;
            viewordersform.Text = "View My Orders";
            viewordersform.Show();
            viewordersform.WindowState = FormWindowState.Maximized;
        }

        private void viewsalesperson_Click(object sender, EventArgs e)
        {
            /*
             * The ViewSalesPerson form is a small dialog, so it can be respawned every call.
             */
            ViewSalesPerson viewsalespersonform = new ViewSalesPerson();
            viewsalespersonform.ShowDialog();
        }

        private void viewReport_Click(object sender, EventArgs e)
        {
            TopCustomers topcustomersform = new TopCustomers();
            topcustomersform.WindowState = FormWindowState.Maximized;
            topcustomersform.ShowDialog();
        }

        private void configuration_Click(object sender, EventArgs e)
        {
            /*
             * The Config form is a small dialog, so it can be respawned every call.
             */
            Configuration configform = new Configuration();
            configform.ShowDialog();
        }

        void syncform_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (syncform.iscancelled)
            {
                try
                {
                    dbmlsync.Stop();
                }
                catch
                {
                }
            }
        }

        void dbmlsync_Message(DbmlsyncCOM.MessageClass msgClass, int msgId, string msg)
        {
            syncform.message.Text += "\r\n" + msg;
        }

        void dbmlsync_DoneExecution()
        {
            statusbarlabel.Text = "Finished Synchronizing.";
            statusbarprogress.Value = statusbarprogress.Minimum;

            if (!dbmlsyncerror)
            {
                syncform.Hide();
                syncform.Dispose();
            }
            else
            {
                MessageBox.Show("Unable to synchronize successfully!\r\nPlease review the message log.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        void dbmlsync_ProgressMessage(string msg)
        {
            statusbarlabel.Text = msg;
            syncform.progressmessage.Text = msg;

            if(msg.Contains("Synchronization Failed"))
            {
                dbmlsyncerror = true;
            }
        }

        void dbmlsync_ProgressIndex(int index, int max)
        {
            if (max != 0)
            {
                statusbarprogress.Maximum = max;
                syncform.progressindex.Maximum = max;
            }
            statusbarprogress.Value = index;
            syncform.progressindex.Value = index;
        }
    }
}