using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using iAnywhere.Data.SQLAnywhere;

namespace AdventureWorksDemo_Remote
{
    public partial class ViewProducts : Form
    {
        public bool dialog
        {
            get
            {
                return _dialog;
            }
            set
            {
                _dialog = value;
            }
        }
        private bool _dialog;
        private SADataReader backgroundreader;
        private System.Threading.EventWaitHandle signal;
        private int productcount;

        public ViewProducts()
        {
            InitializeComponent();
            signal = new System.Threading.EventWaitHandle(false, System.Threading.EventResetMode.AutoReset);
            productcount = 0;
            dialog = false;
        }

        private void ViewProducts_Shown(object sender, EventArgs e)
        {
            search_Click(sender, e);
        }

        private void ViewProducts_Deactivate(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void productlist_ItemActivate(object sender, EventArgs e)
        {
            ViewProductDetails viewproductdetailsform = new ViewProductDetails();
            viewproductdetailsform.productid = (int)productlist.SelectedItems[0].Tag;
            viewproductdetailsform.Show();
        }

        private void search_productnumber_TextChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_productname_TextChanged(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void search_Click(object sender, EventArgs e)
        {
            string sqlstmt;
            string whereclause = "";

            sqlstmt = "SELECT DISTINCT TOP 50 ProductNumber, Name, ListPrice, Production.Product.ProductID FROM Production.Product, Sales.SpecialOfferProduct WHERE Production.Product.ProductID = Sales.SpecialOfferProduct.ProductID";
            if (search_productnumber.Text.Length > 0)
            {
                whereclause += " AND ProductNumber LIKE '%" + search_productnumber.Text + "%'";
            }
            if (search_productname.Text.Length > 0)
            {
                whereclause += " AND Name LIKE '%" + search_productname.Text + "%'";
            }
            if (whereclause.Length > 0)
            {
                sqlstmt += whereclause;
            }

            search.Enabled = false;
            stop.Enabled = true;
            productcount = 0;
            productlist.Items.Clear();
            productlist.Enabled = false;
            productlist.Visible = false;

            if (!dialog)
            {
                (MdiParent as MainForm).statusbarlabel.Text = "Searching for products...";
                (MdiParent as MainForm).statusbarprogress.Value = (MdiParent as MainForm).statusbarprogress.Maximum;
            }

            backgroundprocessor.RunWorkerAsync(sqlstmt);

            if (dialog)
            {
                Font font = new Font(stop.Font, FontStyle.Regular);
                stop.Text = "Stop";
                stop.Font = font;
                stop.DialogResult = DialogResult.None;
            }
        }

        private void stop_Click(object sender, EventArgs e)
        {
            backgroundprocessor.CancelAsync();
        }

        private void productlist_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (dialog)
            {
                Font font = new Font(stop.Font, FontStyle.Bold);
                stop.Text = "Select";
                stop.Font = font;
                stop.Enabled = true;
                stop.DialogResult = DialogResult.Yes;
            }
        }

        private void backgroundprocess(object sender, DoWorkEventArgs e)
        {
            SACommand backgroundcmd;
            string sqlstmt = e.Argument as string;
            bool done = false;

            backgroundcmd = Program.CreateCommand(sqlstmt);
            backgroundreader = backgroundcmd.ExecuteReader();
            
            while (!((sender as BackgroundWorker).CancellationPending) && !done)
            {
                if (backgroundreader.Read())
                {
                    (sender as BackgroundWorker).ReportProgress(1);
                    signal.WaitOne(1000, false);
                }
                else
                {
                    done = true;
                }
            }

            backgroundreader.Close();
            backgroundreader.Dispose();
            backgroundcmd.Dispose();
        }

        private void backgroundprogressupdate(object sender, ProgressChangedEventArgs e)
        {
            productlist.Items.Add(backgroundreader.GetString(0));
            productlist.Items[productcount].SubItems.Add(backgroundreader.GetString(1));
            productlist.Items[productcount].SubItems.Add(backgroundreader.GetDecimal(2).ToString("$###,##0.00"));
            productlist.Items[productcount++].Tag = backgroundreader.GetInt32(3);
            
            if (!dialog)
            {
                (MdiParent as MainForm).statusbarlabel.Text = "Searching for products... Found " + productcount.ToString() + " products.";
            }
            
            signal.Set();
        }

        private void backgroundcompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            productlist.Enabled = true;
            productlist.Visible = true;
            stop.Enabled = false;
            search.Enabled = true;

            if (!dialog)
            {
                (MdiParent as MainForm).statusbarlabel.Text = "Finished. Found " + productcount.ToString() + " products.";
                (MdiParent as MainForm).statusbarprogress.Value = (MdiParent as MainForm).statusbarprogress.Minimum;
            }
        }
    }
}